@@ -55,7 +55,7 @@ def connect(port):
         log.info("Connected.")
         return s
     except:
-        log.exception("Could not connect to: %s", (port,))
+        log.exception("Could not connect to: %s", port)
         raise
 
 
@@ -584,6 +584,11 @@ def main():
         )
 
     processor = _RobotTargetComm(s, debug=debug)
+
+    from robotframework_debug_adapter import global_vars
+
+    global_vars.set_global_robot_target_comm(processor)
+
     reader, writer = processor.start_communication_threads(debug)
 
     if not processor.configuration_done.wait(DEFAULT_TIMEOUT):
