@@ -382,7 +382,7 @@ def _handle_completions(language_server_impl, rf_interpreters_manager, arguments
     interpreter: RfInterpreterServerManager = rf_info_or_dict_error.interpreter
     uri = interpreter.uri
 
-    api = language_server_impl._server_manager.get_others_api_client(uri)
+    api = language_server_impl._server_manager.get_regular_rf_api_client(uri)
     if api is None:
         log.info(
             "Unable to get api client when computing completions (for interactive usage)."
@@ -433,7 +433,7 @@ def _handle_completions(language_server_impl, rf_interpreters_manager, arguments
                     monitor=None,
                 )
         except:
-            log.exception(f"Error computing semantic tokens for arguments: {arguments}")
+            log.exception(f"Error computing completions for arguments: {arguments}")
             return {"suggestions": []}
 
     return rf_info_or_dict_error.ls_thread_pool.submit(run)
