@@ -27,12 +27,17 @@ import typing
 import sys
 from robocorp_ls_core.watchdog_wrapper import IFSObserver
 from robocorp_ls_core.lsp import CodeLensTypedDict
-from robotframework_ls.commands import ROBOT_GET_RFLS_HOME_DIR
+from robotframework_ls.commands import (
+    ROBOT_GET_RFLS_HOME_DIR,
+    ROBOT_START_INDEXING_INTERNAL,
+    ROBOT_WAIT_FULL_TEST_COLLECTION_INTERNAL,
+)
 
 
 log = get_logger(__name__)
 
-LINT_DEBOUNCE_S = 0.4  # 400 ms
+_LINT_DEBOUNCE_IN_SECONDS_LOW = 0.2
+_LINT_DEBOUNCE_IN_SECONDS_HIGH = 0.8
 
 
 class _CurrLintInfo(object):
@@ -42,6 +47,7 @@ class _CurrLintInfo(object):
         lsp_messages,
         doc_uri,
         is_saved,
+        on_finish,
     ) -> None:
         from robocorp_ls_core.lsp import LSPMessages
 
@@ -50,6 +56,7 @@ class _CurrLintInfo(object):
         self.doc_uri = doc_uri
         self.is_saved = is_saved
         self._monitor = Monitor()
+        self._on_finish = on_finish
 
     def __call__(self) -> None:
         from robocorp_ls_core.jsonrpc.exceptions import JsonRpcRequestCancelled
@@ -71,15 +78,19 @@ class _CurrLintInfo(object):
                     diagnostics_msg = message_matcher.msg
                     if diagnostics_msg:
                         found = diagnostics_msg.get("result", [])
+                    self._monitor.check_cancelled()
                     self.lsp_messages.publish_diagnostics(doc_uri, found)
         except JsonRpcRequestCancelled:
-            log.info(f"Cancelled linting: {self.doc_uri}.")
+            log.debug("Cancelled linting: %s.", self.doc_uri)
 
         except SubprocessDiedError:
-            log.info(f"Subprocess exited while linting: {self.doc_uri}.")
+            log.debug("Subprocess exited while linting: %s.", self.doc_uri)
 
         except Exception:
-            log.exception("Error linting.")
+            log.exception("Error linting: %s.", self.doc_uri)
+
+        finally:
+            self._on_finish(self)
 
     def cancel(self):
         self._monitor.cancel()
@@ -95,6 +106,7 @@ def run_in_new_thread(func, thread_name):
 
 class _LintManager(object):
     def __init__(self, server_manager, lsp_messages) -> None:
+        import threading
         from robotframework_ls.server_manager import ServerManager
 
         self._server_manager: ServerManager = server_manager
@@ -102,28 +114,50 @@ class _LintManager(object):
 
         self._next_id = partial(next, itertools.count())
         self._doc_id_to_info: Dict[str, _CurrLintInfo] = {}
+        self._lock = threading.Lock()
 
-    def schedule_lint(self, doc_uri: str, is_saved: bool) -> None:
+    def schedule_lint(self, doc_uri: str, is_saved: bool, timeout: float) -> None:
         self.cancel_lint(doc_uri)
         rf_lint_api_client = self._server_manager.get_lint_rf_api_client(doc_uri)
         if rf_lint_api_client is None:
-            log.info(f"Unable to get lint api for: {doc_uri}")
+            log.info("Unable to get lint api for: %s", doc_uri)
             return
 
+        lock = self._lock
+        doc_id_to_info = self._doc_id_to_info
+
+        def on_finish(curr_info: _CurrLintInfo):
+            # When it's finished, remove it from our references (if it's still
+            # the current one...).
+            try:
+                with lock:
+                    if doc_id_to_info.get(curr_info.doc_uri) is curr_info:
+                        del doc_id_to_info[curr_info.doc_uri]
+            except:
+                log.exception("Unhandled error on lint finish.")
+
+        log.debug("Schedule lint for: %s", doc_uri)
         curr_info = _CurrLintInfo(
-            rf_lint_api_client, self._lsp_messages, doc_uri, is_saved
+            rf_lint_api_client, self._lsp_messages, doc_uri, is_saved, on_finish
         )
+        with self._lock:
+            self._doc_id_to_info[doc_uri] = curr_info
+
         from robocorp_ls_core.timeouts import TimeoutTracker
 
         timeout_tracker = TimeoutTracker.get_singleton()
         timeout_tracker.call_on_timeout(
-            LINT_DEBOUNCE_S, partial(run_in_new_thread, curr_info, f"Lint: {doc_uri}")
+            timeout,
+            partial(run_in_new_thread, curr_info, f"Lint: {doc_uri}"),
         )
 
     def cancel_lint(self, doc_uri: str) -> None:
-        curr_info = self._doc_id_to_info.pop(doc_uri, None)
-        if curr_info is not None:
-            curr_info.cancel()
+        with self._lock:
+            curr_info = self._doc_id_to_info.pop(doc_uri, None)
+            if curr_info is not None:
+                log.debug("Cancel lint for: %s", doc_uri)
+
+                curr_info.cancel()
 
 
 from robocorp_ls_core.command_dispatcher import _CommandDispatcher
@@ -188,14 +222,27 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
             else:
                 watch_impl = "fsnotify"
 
+        log.info(
+            "Using watch implementation: %s (customize with ROBOTFRAMEWORK_LS_WATCH_IMPL environment variable)",
+            watch_impl,
+        )
+
         self._fs_observer = watchdog_wrapper.create_remote_observer(
             watch_impl, (".py", ".libspec", "robot", ".resource")
         )
         remote_observer = typing.cast(RemoteFSObserver, self._fs_observer)
+
         log_file = Setup.options.log_file
-        if not isinstance(log_file, str):
+        verbose = Setup.options.verbose
+        if not isinstance(log_file, str) and log_file is not None:
+            log.critical("Expected %r to be a str (was: %s)", log_file, type(log_file))
             log_file = None
-        remote_observer.start_server(log_file=log_file)
+
+        if not isinstance(verbose, int) and verbose is not None:
+            log.critical("Expected %r to be an int (was: %s)", verbose, type(verbose))
+            verbose = None
+
+        remote_observer.start_server(log_file=log_file, verbose=verbose)
 
         self._server_manager = ServerManager(self._pm, language_server=self)
         self._lint_manager = _LintManager(self._server_manager, self._lsp_messages)
@@ -273,9 +320,8 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
 
         return ret
 
-    def forward_progress_msg(self, msg: dict) -> None:
+    def forward_msg(self, msg: dict) -> None:
         method = msg["method"]
-        assert method == "$/customProgress"
         self._endpoint.notify(method, msg["params"])
 
     @overrides(PythonLanguageServer.capabilities)
@@ -333,7 +379,7 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
                 "full": True,
             },
         }
-        log.info("Server capabilities: %s", server_capabilities)
+        log.debug("Server capabilities: %s", server_capabilities)
         return server_capabilities
 
     def m_workspace__execute_command(self, command: str = "", arguments=()) -> Any:
@@ -351,6 +397,25 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         self._pm.load_plugins_from(Path(directory))
         return True
 
+    @command_dispatcher(ROBOT_START_INDEXING_INTERNAL)
+    def _start_indexing(self, *arguments):
+        self._server_manager.get_regular_rf_api_client("")
+
+    @command_dispatcher(ROBOT_WAIT_FULL_TEST_COLLECTION_INTERNAL)
+    def _wait_for_full_test_collection(self, *arguments):
+        rf_api_client = self._server_manager.get_regular_rf_api_client("")
+        if rf_api_client is not None:
+            func = partial(
+                self._async_api_request_no_doc,
+                rf_api_client,
+                "request_wait_for_full_test_collection",
+            )
+            func = require_monitor(func)
+            return func
+
+        log.info("Unable to wait for first test collection (no api available).")
+        return []
+
     @command_dispatcher("robot.getInternalInfo")
     def _get_internal_info(self, *arguments):
         in_memory_docs = []
@@ -532,8 +597,23 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
     # --- Customized implementation
 
     @overrides(PythonLanguageServer.lint)
-    def lint(self, doc_uri, is_saved) -> None:
-        self._lint_manager.schedule_lint(doc_uri, is_saved)
+    def lint(self, doc_uri, is_saved, content_changes=None) -> None:
+        # We want the timeout to be shorter if it was saved or if the user
+        # typed a new line.
+        timeout = _LINT_DEBOUNCE_IN_SECONDS_LOW
+        if not is_saved:
+            timeout = _LINT_DEBOUNCE_IN_SECONDS_HIGH
+            for change in content_changes:
+                try:
+                    text = change.get("text", "")
+                    if "\n" in text or "\r" in text:
+                        timeout = _LINT_DEBOUNCE_IN_SECONDS_LOW
+                        break
+                except:
+                    log.exception(
+                        "Error computing lint timeout with: %s", content_changes
+                    )
+        self._lint_manager.schedule_lint(doc_uri, is_saved, timeout)
 
     @overrides(PythonLanguageServer.cancel_lint)
     def cancel_lint(self, doc_uri) -> None:
@@ -850,7 +930,7 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
 
         # Note: we want to use the same one used by m_workspace__symbol (to reuse
         # the related caches).
-        rf_api_client = self._server_manager.get_others_api_client(doc_uri)
+        rf_api_client = self._server_manager.get_regular_rf_api_client(doc_uri)
         if rf_api_client is not None:
             func = partial(
                 self._async_api_request,
@@ -888,7 +968,7 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         return func
 
     def m_workspace__symbol(self, query: Optional[str] = None) -> Any:
-        api = self._server_manager.get_others_api_client("")
+        api = self._server_manager.get_regular_rf_api_client("")
         if api is None:
             log.info("Unable to search workspace symbols (no api available).")
             return None
