@@ -18,6 +18,7 @@ from robocorp_ls_core.lsp import (
 from robotframework_ls.impl.protocols import IKeywordFound
 from robocorp_ls_core.watchdog_wrapper import IFSObserver
 import itertools
+import typing
 
 
 log = get_logger(__name__)
@@ -37,10 +38,14 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         libspec_manager=None,
         observer: Optional[IFSObserver] = None,
         pre_generate_libspecs: bool = False,
+        index_workspace: bool = False,
+        collect_tests: bool = False,
     ):
         from robotframework_ls.impl.libspec_manager import LibspecManager
 
         PythonLanguageServer.__init__(self, read_from, write_to)
+        self._index_workspace = index_workspace
+        self._collect_tests = collect_tests
 
         if libspec_manager is None:
             try:
@@ -111,13 +116,18 @@ class RobotFrameworkServerApi(PythonLanguageServer):
     ) -> IWorkspace:
         from robotframework_ls.impl.robot_workspace import RobotWorkspace
 
-        return RobotWorkspace(
+        robot_workspace = RobotWorkspace(
             root_uri,
             fs_observer,
             workspace_folders,
             libspec_manager=self.libspec_manager,
+            index_workspace=self._index_workspace,
+            collect_tests=self._collect_tests,
+            endpoint=self._endpoint,
         )
 
+        return robot_workspace
+
     def m_lint(self, doc_uri):
         if not self._check_min_version((3, 2)):
             from robocorp_ls_core.lsp import Error
@@ -563,6 +573,25 @@ class RobotFrameworkServerApi(PythonLanguageServer):
 
         return list_tests(completion_context)
 
+    def m_wait_for_full_test_collection(self):
+        func = partial(self._threaded_wait_for_full_test_collection)
+        func = require_monitor(func)
+        return func
+
+    def _threaded_wait_for_full_test_collection(self, monitor: IMonitor) -> bool:
+        from robotframework_ls.impl.robot_workspace import RobotWorkspace
+
+        workspace = self.workspace
+        if not workspace:
+            log.info("Workspace still not initialized.")
+            return False
+        ws = typing.cast(RobotWorkspace, workspace)
+        workspace_indexer = ws.workspace_indexer
+        if not workspace_indexer:
+            raise RuntimeError("WorkspaceIndexer not available (None).")
+        workspace_indexer.wait_for_full_test_collection()
+        return True
+
     def m_hover(self, doc_uri: str, line: int, col: int):
         func = partial(self._threaded_hover, doc_uri, line, col)
         func = require_monitor(func)
@@ -839,7 +868,13 @@ class RobotFrameworkServerApi(PythonLanguageServer):
     def m_shutdown(self, **_kwargs):
         PythonLanguageServer.m_shutdown(self, **_kwargs)
         self.libspec_manager.dispose()
+        workspace = self._workspace
+        if workspace is not None:
+            workspace.dispose()
 
     def m_exit(self, **_kwargs):
         PythonLanguageServer.m_exit(self, **_kwargs)
         self.libspec_manager.dispose()
+        workspace = self._workspace
+        if workspace is not None:
+            workspace.dispose()
