@@ -709,6 +709,7 @@ class %(name)s(BaseSchema):
 
     contents = []
     contents.append("# coding: utf-8")
+    contents.append("# fmt: off")
     contents.append("# Automatically generated code.")
     contents.append("# Do not edit manually.")
     contents.append("# Generated by running: %s" % os.path.basename(__file__))
@@ -719,6 +720,8 @@ class %(name)s(BaseSchema):
     for class_to_generate in classes_to_generate.values():
         contents.append(class_template % class_to_generate)
 
+    contents.append("# fmt: on")
+    contents.append("")
     parent_dir = os.path.dirname(__file__)
     schema = os.path.join(parent_dir, "dap_schema.py")
     with open(schema, "w", encoding="utf-8") as stream:
