@@ -27,6 +27,8 @@ if typing.TYPE_CHECKING:
     from robocorp_ls_core.lsp import TextDocumentTypedDict
     from robocorp_ls_core.lsp import ResponseTypedDict
     from robocorp_ls_core.lsp import CodeLensTypedDict
+    from robocorp_ls_core.lsp import RangeTypedDict
+    from robocorp_ls_core.callbacks import Callback
 
 # Hack so that we don't break the runtime on versions prior to Python 3.8.
 if sys.version_info[:2] < (3, 8):
@@ -186,6 +188,9 @@ class IRequestHandler(Protocol):
 
 
 class ILanguageServerClientBase(IRequestCancellable, Protocol):
+
+    on_message: "Callback"
+
     def request_async(self, contents: Dict) -> Optional[IIdMessageMatcher]:
         """
         API which allows to wait for the message to complete.
@@ -352,11 +357,27 @@ class IRobotFrameworkApiClient(ILanguageServerClientBase, Protocol):
     def settings(self, settings: Dict):
         pass
 
+    def request_wait_for_full_test_collection(self) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
 
 class ITestInfoTypedDict(TypedDict):
     uri: str
     path: str
     name: str
+    range: "RangeTypedDict"
+
+
+class ITestInfoFromSymbolsCacheTypedDict(TypedDict):
+    name: str
+    range: "RangeTypedDict"
+
+
+class ITestInfoFromUriTypedDict(TypedDict):
+    uri: str
+    testInfo: List[ITestInfoFromSymbolsCacheTypedDict]
 
 
 class ILanguageServerClient(ILanguageServerClientBase, Protocol):
@@ -608,6 +629,8 @@ class IDocument(Protocol):
     version: Optional[str]
     path: str
 
+    immutable: bool
+
     def selection(self, line: int, col: int) -> IDocumentSelection:
         pass
 
@@ -653,6 +676,8 @@ class IWorkspaceFolder(Protocol):
 
 
 class IWorkspace(Protocol):
+    on_file_changed: "Callback"
+
     @property
     def root_path(self):
         pass
@@ -691,6 +716,9 @@ class IWorkspace(Protocol):
         Retuns the folders which are set as workspace folders.
         """
 
+    def dispose(self):
+        pass
+
 
 class ITimeoutHandle(Protocol):
     def exec_on_timeout(self):
