@@ -36,7 +36,6 @@ from collections import namedtuple
 import time
 from robocorp_ls_core.watchdog_wrapper import IFSObserver
 
-
 log = get_logger(__name__)
 
 _FileMTimeInfo = namedtuple("_FileMTimeInfo", "st_mtime, st_size")
@@ -56,6 +55,7 @@ class _VirtualFSThread(threading.Thread):
     def __init__(self, virtual_fs):
         from robocorp_ls_core.watchdog_wrapper import IFSWatch
         from robocorp_ls_core import load_ignored_dirs
+        from robocorp_ls_core.callbacks import Callback
 
         threading.Thread.__init__(self)
         self.daemon = True
@@ -74,6 +74,7 @@ class _VirtualFSThread(threading.Thread):
         self._fs_watch: Optional[IFSWatch] = None
         self._dirs_changed = set()
         self._trigger_loop = threading.Event()
+        self.on_file_changed = Callback()
 
     def _check_need_sleep(self):
         last_sleep = self._last_sleep
@@ -202,6 +203,7 @@ class _VirtualFSThread(threading.Thread):
         changed_dir = os.path.dirname(src_path)
         self._dirs_changed.add(changed_dir)
         self._trigger_loop.set()
+        self.on_file_changed(src_path)
 
     def dispose(self):
         fs_watch = self._fs_watch
@@ -237,6 +239,7 @@ class _VirtualFS(object):
         # Do initial scan and then start tracking changes.
         self._virtual_fsthread = _VirtualFSThread(self)
         self._virtual_fsthread.start()
+        self.on_file_changed = self._virtual_fsthread.on_file_changed
 
     def wait_for_check_done(self, timeout):
         self._virtual_fsthread.wait_for_check_done(timeout)
@@ -276,6 +279,7 @@ class _WorkspaceFolderWithVirtualFS(object):
         self._vs: _VirtualFS = _VirtualFS(
             self.path, track_file_extensions, fs_observer=fs_observer
         )
+        self.on_file_changed = self._vs.on_file_changed
 
     def _iter_all_doc_uris(self, extensions: Tuple[str, ...]) -> Iterable[str]:
         """
@@ -313,6 +317,7 @@ class Workspace(object):
         track_file_extensions=(".robot", ".resource"),
     ) -> None:
         from robocorp_ls_core.lsp import WorkspaceFolder
+        from robocorp_ls_core.callbacks import Callback
 
         self._main_thread = threading.current_thread()
 
@@ -329,6 +334,8 @@ class Workspace(object):
         # Contains the docs pointing to the filesystem.
         self._filesystem_docs: Dict[str, IDocument] = {}
 
+        self.on_file_changed = Callback()
+
         if workspace_folders is not None:
             for folder in workspace_folders:
                 self.add_folder(folder)
@@ -345,8 +352,16 @@ class Workspace(object):
                 f"Mutating the workspace can only be done at the thread: {self._main_thread}. Current thread: {curr_thread}"
             )
 
-    def _create_document(self, doc_uri, source=None, version=None):
-        return Document(doc_uri, source=source, version=version)
+    def _create_document(
+        self, doc_uri, source=None, version=None, force_load_source=False
+    ):
+        return Document(
+            doc_uri,
+            source=source,
+            version=version,
+            mutate_thread=self._main_thread,
+            force_load_source=force_load_source,
+        )
 
     def add_folder(self, folder: IWorkspaceFolder):
         """
@@ -361,6 +376,7 @@ class Workspace(object):
                 track_file_extensions=self._track_file_extensions,
                 fs_observer=self._fs_observer,
             )
+            folder.on_file_changed.register(self.on_file_changed)
             folders[folder.uri] = folder
             self._folders = folders
 
@@ -369,6 +385,7 @@ class Workspace(object):
         if folder_uri in self._folders:
             folders = self._folders.copy()
             folder = folders.pop(folder_uri)
+            folder.on_file_changed.unregister(self.on_file_changed)
             folder.dispose()
             self._folders = folders
 
@@ -413,13 +430,14 @@ class Workspace(object):
                     doc = None
 
             if doc is None:
-                doc = self._create_document(doc_uri)
                 try:
-                    _source = doc.source  # Force loading current contents
+                    doc = self._create_document(doc_uri, force_load_source=True)
                 except:
+                    log.info("Unable to load contents from: %s", doc_uri)
                     # Unable to load contents: file does not exist.
                     doc = None
                 else:
+                    doc.immutable = True
                     self._filesystem_docs[doc_uri] = doc
 
         return doc
@@ -440,6 +458,9 @@ class Workspace(object):
         try:
             # In case the initial text wasn't passed, try to load it from source.
             # If it doesn't work, set the initial source as empty.
+            # Note that we already checked the thread in this function, so, we
+            # don't need to force it to be gotten in the constructor (which
+            # could raise an exception).
             _source = doc.source
         except:
             doc.source = ""
@@ -463,7 +484,7 @@ class Workspace(object):
 
     def update_document(
         self, text_doc: TextDocumentItem, change: TextDocumentContentChangeEvent
-    ):
+    ) -> IDocument:
         self._check_in_mutate_thread()
         doc_uri = text_doc["uri"]
         doc = self._docs[doc_uri]
@@ -473,6 +494,7 @@ class Workspace(object):
         new_doc = self._create_document(doc_uri, doc.source, text_doc["version"])
         new_doc.apply_change(change)
         self._docs[doc_uri] = new_doc
+        return new_doc
 
     def iter_all_doc_uris_in_workspace(
         self, extensions: Tuple[str, ...]
@@ -488,6 +510,9 @@ class Workspace(object):
         for folder in folders:
             yield from folder._iter_all_doc_uris(extensions)
 
+    def dispose(self):
+        pass
+
     def __typecheckself__(self) -> None:
         from robocorp_ls_core.protocols import check_implements
 
@@ -504,8 +529,18 @@ class Document(object):
     once, but that should not corrupt internal structures).
     """
 
-    def __init__(self, uri: str, source=None, version: Optional[str] = None):
+    def __init__(
+        self,
+        uri: str,
+        source=None,
+        version: Optional[str] = None,
+        *,
+        mutate_thread=None,
+        force_load_source=False,
+    ):
+        # During construction, set the mutate thread to the current thread.
         self._main_thread = threading.current_thread()
+        self.immutable = False
 
         self.uri = uri
         self.version = version
@@ -517,6 +552,15 @@ class Document(object):
         # Only set when the source is read from disk.
         self._source_mtime = -1
 
+        if force_load_source:
+            # Just accessing should be ok to load the source.
+            _ = self.source
+
+        if mutate_thread is not None:
+            # After construction it may only be mutated by the mutate thread if
+            # specified.
+            self._main_thread = mutate_thread
+
     def _check_in_mutate_thread(self):
         curr_thread = threading.current_thread()
         if self._main_thread is not curr_thread:
@@ -548,6 +592,10 @@ class Document(object):
     def _source(self, source: str) -> None:
         # i.e.: when the source is set, reset the lines.
         self._check_in_mutate_thread()
+        if self.immutable:
+            raise RuntimeError(
+                "This document is immutable, so, its source cannot be changed."
+            )
         self.__source = source
         self._clear_caches()
 
